package org.simplericity.serberuhs.keytab;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 *
 */
public class KeytabParser
{
    public List <KeytabInfo> parse(InputStream resource) {

        try {

            DataInputStream is = new DataInputStream(byteArrayStream(resource));

            List<KeytabInfo> infos = new ArrayList<KeytabInfo>();

            short fileFormatVersion = is.readShort();

            while(is.available() > 0) {
                int size = is.readInt();
                int byteCount = 0;

                int components = is.readShort();
                byteCount +=2;
                byte[] realm = readOctetString(is);
                byteCount += realm.length +2;
                String serviceType = null;
                String host = null;
                for(int  i = 0; i < components; i++) {
                    byte[] comp = readOctetString(is);
                    byteCount += comp.length +2;
                    String compString = new String(comp, "utf-8");
                    if(i == 0) {
                        serviceType = compString;
                    } else if(i == 1) {
                        host = compString;
                    }
                }
                int nameType = is.readInt();
                byteCount += 4;
                int timestamp = is.readInt();
                byteCount += 4;
                int kvno = is.read();
                byteCount += 1;

                int keyBlockType = is.readShort();
                byteCount += 2;
                byte[] key = readOctetString(is);
                byteCount += key.length +2;
                if(size > byteCount) {
                    kvno = is.readInt();
                }

                infos.add(new DefaultKeytabInfo(new String(realm, "utf-8"), serviceType, host, kvno));
            }
            return infos;
        } catch (IOException e) {
            throw new RuntimeException(e);
        }


    }

    private InputStream byteArrayStream(InputStream resource) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();

        byte[] buffer = new byte[1024];

        int r = 0;

        while((r = resource.read(buffer)) > 0) {
            out.write(buffer, 0, r);
        }

        resource.close();

        return new ByteArrayInputStream(out.toByteArray());
    }

    private byte[] readOctetString(DataInputStream is) throws IOException {
        short size = is.readShort();
        byte[] bytes = new byte[size];

        is.read(bytes);
        return bytes;
    }

    private static class DefaultKeytabInfo implements KeytabInfo {
        private final String realm;
        private final String servicePart;
        private final String hostPart;
        private final int kvno;

        public DefaultKeytabInfo(String realm, String servicePart, String hostPart, int kvno) {
            this.realm = realm;
            this.servicePart = servicePart;
            this.hostPart = hostPart;
            this.kvno = kvno;
        }

        public String getHostPart() {
            return hostPart;
        }

        public int getKvno() {
            return kvno;
        }

        public String getRealm() {
            return realm;
        }

        public String getServicePart() {
            return servicePart;
        }
    }
}
